local IsNiGHTS = CrashAddon.IsNiGHTS
local JumpManager = CrashAddon.JumpManager
local ObjectSpawner = CrashAddon.ObjectSpawner

local SuperColors = {
	[SKINCOLOR_SUPERORANGE1] = SKINCOLOR_SUPERGOLD1,
	[SKINCOLOR_SUPERORANGE2] = SKINCOLOR_SUPERGOLD2,
	[SKINCOLOR_SUPERORANGE3] = SKINCOLOR_SUPERGOLD3,
	[SKINCOLOR_SUPERORANGE4] = SKINCOLOR_SUPERGOLD4,
	[SKINCOLOR_SUPERORANGE5] = SKINCOLOR_SUPERGOLD5
}

local SuperStates = {
	[S_PLAY_SUPER_TRANS1] = true,
	[S_PLAY_SUPER_TRANS2] = true,
	[S_PLAY_SUPER_TRANS3] = true,
	[S_PLAY_SUPER_TRANS4] = true,
	[S_PLAY_SUPER_TRANS5] = true,
	[S_PLAY_SUPER_TRANS6] = true
}

//this handles Aku Aku's behavior
local function AkuAkuThinker(mo)
	//failsafe
	if not mo.target or not mo.target.valid or not mo.target.player or 
			IsNiGHTS(mo.target.player) or mo.target.player.playerstate ==
			PST_DEAD
		P_RemoveMobj(mo)
		return
	end
	local akulevel = mo.target.player.crashban.akulevel
	if akulevel <= 0
		P_RemoveMobj(mo)
		return
	end
	//this determines sound cues
	if mo.currentlevel == nil
		mo.currentlevel = akulevel
	end
	if mo.currentlevel != akulevel
		if mo.currentlevel < akulevel and akulevel != 4
			S_StartSound(mo.target, sfx_akuspn)
		end
		mo.currentlevel = akulevel
	end
	//this makes aku aku semi-transparent if the player is in first-person
	if displayplayer == mo.target.player
		if camera.chase or (camera2 != nil and camera2.chase)
			mo.frame = $ & ~FF_TRANS80
		else
			mo.frame = $ | FF_TRANS80
		end
	end
	mo.scale = mo.target.scale
	mo.shadowscale = mo.scale - mo.scale / 4
	mo.eflags = mo.target.eflags
	if SuperStates[mo.target.state]
		mo.flags2 = $ | MF2_DONTDRAW
	else
		mo.flags2 = $ & ~MF2_DONTDRAW
	end
	local isSuper = mo.target.player.powers[pw_super] and 
			mo.target.player.crashban.supertrans
	//this changes Aku Aku's color based on aku level
	if akulevel > 1
		if isSuper
			mo.colorized = true
			mo.color = SuperColors[mo.target.color] or SKINCOLOR_GOLDENROD
			mo.frame = $ | FF_FULLBRIGHT
		elseif akulevel <= 3
			mo.colorized = true
			mo.frame = $ & ~FF_FULLBRIGHT
			if leveltime % 3 == 0
				if akulevel > 2 and mo.color == SKINCOLOR_GOLDENROD
					mo.color = SKINCOLOR_SUPERGOLD2
				else
					mo.color = SKINCOLOR_GOLDENROD
				end
			end
		end
	else
		mo.colorized = false
	end
	local invincibility = mo.target.player.powers[pw_invulnerability]
	//this makes Aku Aku orbit around the player
	if akulevel < 3
		if invincibility or isSuper
			mo.target.player.crashban.akulevel = 3
		end
		mo.threshold = 40 * mo.scale
		A_UnidusBall(mo)
		A_FaceTarget(mo)
		mo.angle = $ + ANGLE_180
		//this is all the fancy visual effects
		if not (mo.frame & FF_TRANS80) and leveltime % 2 == 0
			local akughost = P_SpawnGhostMobj(mo)
			akughost.colorized = true
			if akulevel > 1
				akughost.color = SKINCOLOR_GOLDENROD
				akughost.frame = $ | FF_ADD
			else
				akughost.color = mo.target.color
				akughost.frame = $ & ~FF_ADD
			end
		end
		if akulevel > 1 and leveltime % 7 == 0
			local akusparks = ObjectSpawner(mo, MT_IVSP)
			akusparks.colorized = true
			akusparks.frame = $ | FF_ADD | FF_TRANS70
			akusparks.color = SKINCOLOR_GOLDENROD
		end
	//this gives Crash invincibility and moves Aku Aku once max level is reached
	elseif akulevel == 3
		A_ForceStop(mo, 0)
		if not invincibility and not isSuper
			if not mo.invin
				if not isSuper
					A_Invincibility(mo)
				end
				mo.invin = true
			elseif not isSuper
				mo.invin = false
				mo.target.player.crashban.akulevel = 2
				S_StartSound(mo.target, sfx_akuded)
			end
		elseif not mo.invin
			mo.invin = true
		end
		//this makes aku aku be in front of crash's face
		mo.angle = mo.target.player.drawangle
		local akux = mo.target.x + P_ReturnThrustX(mo, mo.angle, 25 * mo.scale)
		local akuy = mo.target.y + P_ReturnThrustY(mo, mo.angle, 25 * mo.scale)
		local akuheight
		if mo.eflags & MFE_VERTICALFLIP
			akuheight = mo.target.height / 2 * -1 + mo.target.height / 3 + 
					mo.target.height / 10
		else
			akuheight = mo.target.height - mo.target.height / 3 - 
					mo.target.height / 10
		end
		P_MoveOrigin(mo, akux, akuy, akuheight + mo.target.z)
	//this handles the behavior during the super transformation sequence
	else
		local transtimer = mo.target.player.crashban.transtimer
		if transtimer == 2 * TICRATE + TICRATE / 3 - 1
			local akuheight
			if mo.eflags & MFE_VERTICALFLIP
				akuheight = mo.target.height / 2 * -1
			else
				akuheight = mo.target.height
			end
			P_MoveOrigin(mo, mo.target.x, mo.target.y, akuheight + mo.target.z)
		elseif transtimer > TICRATE / 2
			JumpManager(mo, 1 * FRACUNIT + FRACUNIT / 2)
			mo.angle = $ + ANGLE_22h
		elseif transtimer >= 3
			if transtimer == TICRATE / 2
				S_StartSound(mo.target, sfx_akuspn)
				local akusparks = ObjectSpawner(mo, MT_IVSP)
				akusparks.colorized = true
				akusparks.frame = $ | FF_ADD
				akusparks.color = SKINCOLOR_GOLDENROD
			end
			A_ForceStop(mo, 0)
			mo.angle = mo.target.player.drawangle + ANGLE_180
			if leveltime % 2 == 0
				if mo.colorized
					mo.colorized = false
				else
					mo.colorized = true
					mo.color = SKINCOLOR_SUPERGOLD1
				end
			end
		else
			JumpManager(mo, -15 * FRACUNIT)
		end
	end
end

//hooks
addHook("MobjThinker", AkuAkuThinker, MT_AKUAKU)